// Copyright 2020 T-Systems International GmbH
import app from 'app';
import eventBus from 'js/eventBus';
import appCtxSvc from 'js/appCtxService';
import soaSvc from 'soa/kernel/soaService';
import messagingSvc from 'js/messagingService';
import adapterSvc from 'js/adapterService';
import navigationSvc from 'js/navigationService';

/**
 * Service for the command sendToCatia.
 *
 * @module js/sendToCatiaService
 */
var exports = {};

const CODE = "code";
const ERROR = "Error";
const MESSAGE = "Message";
const ATTRIBUTE = "Attribute";
const ID = "id";
const NAME = "name";
const VALUE = "value";
const CONTENT_TYPE_XML = "application/xml";

/**
 * Close active Info Panel
 */
let closeCurrentPanel = () => {
	//To close the panel
	var eventData = { source: 'toolAndInfoPanel' };
	eventBus.publish('complete', eventData);
};


/**
 * Send a Request to CMI
 * @param {*} requestData - Data to send with HTTP Post request can be an xml document or string
 * @param {string} contectType - Content Type. Default is 'text/plain' for string requestData. It depends on requestData type. In case of XML it must be set as 'application/xml'
 * @param {boolean} showAlert - If this function should show alerts, default is false
 * @param {boolean} closePanel - Decision to close Panel after successful request, default is false
 * @returns {Promise} - Promis with resolved XML Response from call or error messages in reject
 * 
 */
let makeRequestToCMI = function (requestData, showAlert = false, closePanel = false, contentType = 'text/plain', useXCIUrl = false) {

	return new Promise((resolve, reject) => {
		let prefName = 'CMIRII_SERVER_URL';
		if (useXCIUrl) {
			prefName = 'XCI_SERVER_URL';
		}
		let cmiriiServerUrl = null;
		let responseXmlDocument = null;

		// get Teamcenter preference
		soaSvc.postUnchecked('Administration-2012-09-PreferenceManagement', 'getPreferences', {
			preferenceNames: [prefName],
			includePreferenceDescriptions: false
		}).then(
			function (result) {
				if (result) {
					if (result.partialErrors || result.PartialErrors) {
						if (showAlert) {
							messagingSvc.showWarning("partialErrors" + result.partialErrors + " " + result.PartialErrors);
						}
					}

					// get the Teamcenter preference of the result
					if (result.response[0] != undefined) {
						if (result.response[0].values.values.length > 0 && result.response[0].values.values[0] !== '') {
							cmiriiServerUrl = result.response[0].values.values[0];
							if (useXCIUrl) {
								cmiriiServerUrl = cmiriiServerUrl + '/XCIRequest';
							}
						}
					}
				}

				// check if the Teamcenter preference is filled
				if (cmiriiServerUrl == null || cmiriiServerUrl == "") {
					let msg = `Please set Teamcenter Preference ${prefName}.`;
					if (showAlert) {
						messagingSvc.showWarning(msg);
					}
					return reject(msg);
				}

				// create XMLHttpRequest
				let http = new XMLHttpRequest();

				http.open("POST", cmiriiServerUrl, true); // true -> Async

				http.onload = function () {
				};

				http.onerror = function () {
					// write to html console
					let message = `Error: Could not send request to ${cmiriiServerUrl}.`;
					if (showAlert) {
						messagingSvc.showError(message);
					}
					reject(message);
				};

				http.onreadystatechange = function () {
					if (http.readyState == 4 && http.status == 200) {
						if (http.responseXML) {
							responseXmlDocument = http.responseXML;
						}
						else if (http.responseText) {
							// parse xml content to xmlDocument
							let response = http.responseText;
							var parser = new DOMParser();
							responseXmlDocument = parser.parseFromString(response, "application/xml");
						}
						else {
							// parse xml content to xmlDocument
							let response = http.response;
							var parser = new DOMParser();
							responseXmlDocument = parser.parseFromString(response, "application/xml");
						}

						//for debug
						// const serializer = new XMLSerializer();
						// const xmlStr = serializer.serializeToString(responseXmlDocument);
						// console.info(xmlStr);

						let msgElements = responseXmlDocument.getElementsByTagName(MESSAGE);
						let messages = [];
						for (const element of msgElements) {
							messages.push(element.childNodes[0].nodeValue);
						}

						if (messages.length > 0) {
							if (showAlert) {
								messagingSvc.showInfo(messages.join());
							}
						}

						// parse xml response and check for error tags
						let errors = responseXmlDocument.getElementsByTagName(ERROR);
						if (errors.length > 0) {
							// show error dialog
							let statusMessages = [];
							for (let x = 0; x < errors.length; x++) {
								// for the value
								statusMessages.push(errors.item(x).innerHTML + " [" + errors.item(x).getAttribute(CODE) + "]");
							}
							let displayResponse = statusMessages.join(', \r\n');
							if (showAlert) {
								messagingSvc.showError('An error occured:\n' + displayResponse);
							}
							reject(displayResponse);
						}

						resolve(responseXmlDocument);
					}
				};

				http.setRequestHeader('Content-type', `${contentType}; charset=utf-8`);

				//for debug
				// const serializer = new XMLSerializer();
				// const xmlStr = serializer.serializeToString(inputXmlDocument);
				// console.info(xmlStr);

				try {
					// send data
					if (contentType === 'text/plain') {
						http.send(JSON.stringify(requestData));
					} else {
						http.send(requestData);
					}

					//To close the panel
					if (closePanel) {
						closeCurrentPanel();
					}

				} catch (e) {
					// Network error, etc.
					let msg = http.response + "--Failed to send Request: " + e;
					if (showAlert) {
						messagingSvc.showError(msg);
					}
					reject(msg);
				}
			},
			function (reason) {
				messagingSvc.showError(`Failed to get Teamcenter Preference ${prefName}: ${reason.toString()}`);
				cmiriiServerUrl = null;
			}
		);
	});
}


/**
* This function gets the rules from the response and returns an array.
*/
export let getRules = function (response) {
	var outputRules = [];

	if (response.ServiceData.modelObjects != undefined) {
		// get length of model objects in the response
		var length = Object.keys(response.ServiceData.modelObjects).length;
		for (var i = 0; i < length; i++) {
			var assignedRule = {};
			var uid = Object.keys(response.ServiceData.modelObjects)[i];
			var displayValue = response.ServiceData.modelObjects[uid].props.object_string.uiValues[0];

			outputRules.push({
				propInternalValue: uid,
				propDisplayValue: displayValue
			});
		}
	}
	return outputRules;
};

/**
* This function gets the selected item revisions and returns an array.
*/
export let getItemRevisions = function () {
	var outputItemRevisions = [];

	// get selected item revisions
	var items = appCtxSvc.ctx.mselected;
	for (var i = 0; i < items.length; i++) {
		var assignedItemRevision = {};

		// get uid and type to add them to the return array
		assignedItemRevision["uid"] = items[i].uid;
		assignedItemRevision["type"] = items[i].type;
		outputItemRevisions.push(assignedItemRevision);
	}
	return outputItemRevisions;
};

/**
* This function gets the uid for a given object.
*/
export let getUid = function (db, list, revision) {
	var outputUid = undefined;
	for (var i = 0; i < revision.length; i++) {
		if (list.uiValue == revision[i].propDisplayValue) {
			outputUid = revision[i].uid;
		}
	}
	return outputUid;
};

/**
* This function sends the request.
*/
export let sendToCatiaTree = function (data) {
	// get values of the dialog:
	// get object string of the Item Revision
	var objectString = data.sendToCatia_txtObjectString.dbValue;

	// get uid of the Item Revision
	var uid = data.sendToCatia_txtUid.dbValue;

	// get uid of the Revision Rule; else "undefined"
	var revisionRuleUid = data.sendToCatia_txtRevisionRule.dbValue;

	// get uid of the Variant Rule; else "undefined"
	var variantRuleUid = data.sendToCatia_txtVariantRule.dbValue;

	// check if only Variant Rule is selected: then error message
	if (variantRuleUid && variantRuleUid != "undefined" && (revisionRuleUid == null || revisionRuleUid == "" || revisionRuleUid == "undefined")) {
		messagingSvc.showWarning("Please select a Revision Rule, too.");
		return;
	}

	// requestData for the rccmicl.jar
	var requestData = "x3SendToCatia|" + "xfile.cmi" + "|" + uid + "|" + revisionRuleUid + "|" + variantRuleUid;


	makeRequestToCMI(requestData, false, true).then((responseXml) => {

		// if(responseXml){
		// 	//for debug
		// 	const serializer = new XMLSerializer();
		// 	const xmlStr = serializer.serializeToString(responseXml);
		// 	console.log(xmlStr);

		// }
		// else{
		// 	console.log("Response XML is null...");
		// }
		messagingSvc.showInfo(`Send to CATIA of ${objectString} finished.`);
		messagingSvc.showInfo("You can now switch to CATIA!");

	}).catch((errMsg) => {
		messagingSvc.showError(errMsg);
	});

};

/**
* This function sends the request.
*/
export let sendToCatiaFolder = function (data) {
	// get values of the dialog:
	// get object string of the Item Revision
	var objectString = data.sendToCatia_txtObjectString.dbValue;

	// get uid of the Item Revision
	var uid = data.sendToCatia_txtUid.dbValue;

	// get uid of the Revision Rule; else "undefined"
	var revisionRuleUid = data.sendToCatia_listRevisionRules.dbValue;
	// var revisionRuleUid = exports.getUid(data, data.sendToCatia_listRevisionRules, data.revisionRules) != undefined ? exports.getUid(data, data.sendToCatia_listRevisionRules, data.revisionRules) : "undefined";

	// get uid of the Variant Rule; else "undefined"
	var variantRuleUid = data.sendToCatia_listVariantRules.dbValue;
	// var variantRuleUid = exports.getUid(data, data.sendToCatia_listVariantRules, data.variantRules) != undefined ? exports.getUid(data, data.sendToCatia_listVariantRules, data.variantRules): "undefined";

	// check if only Variant Rule is selected: then error message
	if (variantRuleUid && variantRuleUid != "undefined" && (revisionRuleUid == null || revisionRuleUid == "" || revisionRuleUid == "undefined")) {
		messagingSvc.showWarning("Please select a Revision Rule, too.");
		return;
	}

	// requestData for the rccmicl.jar
	var requestData = "x3SendToCatia|" + "xfile.cmi" + "|" + uid + "|" + revisionRuleUid + "|" + variantRuleUid;


	makeRequestToCMI(requestData, false, true).then((responseXml) => {

		// if(responseXml){
		// 	//for debug
		// 	const serializer = new XMLSerializer();
		// 	const xmlStr = serializer.serializeToString(responseXml);
		// 	console.log(xmlStr);

		// }
		// else{
		// 	console.log("Response XML is null...");
		// }
		messagingSvc.showInfo(`Send to CATIA of ${objectString} finished.`);
		messagingSvc.showInfo("You can now switch to CATIA!");

	}).catch((errMsg) => {
		messagingSvc.showError(errMsg);
	});

};


export default exports = {
	getRules,
	getItemRevisions,
	getUid,
	sendToCatiaTree,
	sendToCatiaFolder
};

app.factory('sendToCatiaService', () => exports);